# Discovering frames of arguments

This project is part of the course "Softwareprojekt WS 2020/21" of the Computational Linguistics program at Heidelberg University. This project is supervised by Prof. Dr. Anette Frank. <br>
In this project, we take the approach of [Paul&Frank(2019)](https://www.aclweb.org/anthology/N19-1368/) and extend it by the approach of filtering human needs on argumentative texts. 

## Getting Started

These instructions will get you a copy of the project up and running on your local machine for development and testing purposes. 

### Prerequisites

What things you need to install and how to install them

```
- python3.6
- nltk
- python igraph
- Tensorflow 1.15.0 
- Keras 2.2.4
- Tensorflow-hub
- ConceptNet 5.6.0
- matplotlib

```

### Installing
Use the package manager [pip](https://pip.pypa.io/en/stable/) to install

```
- pip install tensorflow
- pip install tensorflow-hub
- pip install nltk
- pip install python-igraph
- pip install spacy 
- pip install sklearn
- pip install configparser
- pip install matplotlib
```

## Data Preperation

To execute the project, you must first select a dataset. <br>
A subset of the dataset must be annotated manually to act as the gold standard. <br>
For the annotation we used Maslow and Reiss motives. Maslow's hierarchy of needs is a motivational theory in psychology comprising a five-tier model of human needs, often depicted as hierarchical levels within a pyramid. Reiss builds on this pyramid, adding finer terms to each category. Each four of us annotated one-hundred of the four-hundred essays with a Maslow and a Reiss motive. We then used Fleiss Kappa for calculation of the Inter Annotater Agreement. <br> Afterwards the data has to be prepared accordingly, both train and test file need to be in a certain format.

## Steps to get started <br>

### Pre-work
First of all, two code files must be executed: ***Comparer.py*** and ***Readhumans.py***. The *Comparer.py* requires the concept-net-assertions-5.6.0 and the selected dataset. The output is a concept-filtered and lemmatized set of words from each input set. When executing the Comparer code, the concrete path where the dataset is located must be specified. After that ontology_create has to be executed for the list with the concepts. Then the sets are split and lemmatized. Run through matching_dicts for final result with concepts. <br> Second, the *Readhumans.py* file is executed. As training set we use the ROCStories dev-set. The output is a file with the individual components of the set (e.g. file name). To choose the right directory in line 338 the file has to be edited specificly (dev -> motiv -> allcharlines).
<br> 

### Start coding
We used the steps provided by Debjit Paul from Ranking and Selecting Multi-Hop Knowledge Paths to Better Predict Human Needs (NAACL 2019)'. However, we had to modify and adapt these steps, which is why our given steps are slightly different from his. The code also differs as the oiginal code either didn't run or didn't work for our project, so we made alterations accordingly. <br>

### *Step 1:* Construct ConceptNet into a graph
Prerequisite for this step is the previous download of the concept-net-assertions-5.6.0.csz.gz.
> **Input:**
>> Inputfile: path to unzipped conceptnet-assertions-5.6.0.csv
>
> 
>
> **Output:** is the concept_graph_full
```python
python src/graph_model/conceptnet2graph.py /"path_to_unziped_conceptnet-assertions-5.6.0.csv_File"
```


### *Step 2:* Construct subgraph per sentence for train and test files

In this step we construct the subgraph for each sentence. <br>
> **Input:** 
>> Inputfile: Path to train or test data <br>
>> Graphpath path to ConceptNet Graph <br>
>> Outputpath: spezifies path to folder where the output will be saved to
    
>
> **Output:** 
>> 
>> Output: folder with file for each sentence containing constructed subgraph 
>
> **Purpose:** --dev | --train | --test

```python
python src/graph_model/make_sub_graph_server.py "inputfile" "graphpath" "outputpath" "--purpose" purpose 
```
e.g. python src/graph_model/make_sub_graph_server.py .\training_neu.txt .\concept_graph_full .\output --purpose train <br> 

*Inputformat:*
~~~~
"unique_ID" + "\t" + "HumanNeed | None" + "\t" + "sentence" + "\t" + "concepts" +"\n"
~~~~
Inputfiles need to follow the aforementioned Inputformat

### *Step 3:* Extracting relevant knowledge paths from subgraphs
> **Input**
>> Inputpath: path to the same inputfile that was used in step 2 <br>
>> Input: path to the output of step 2, i.e. folder with subgraphs <br>
>> Outputpath: path to txt.file where extracted knowledgepaths can be saved <br>
>
> **Output**

>> Output: .txtfile with knowledgepaths for each sentence

```python
python src/graph_model/extract_path.py 
```

## Authors

* Jasikka Pirapakaran - pirapakaran@cl.uni-heidelberg.de 
* Ufkun Menderes - menderes@cl.uni-heidelberg.de
* Dorian Heide - heide@cl.uni-heidelberg.de 
* Stefanie Kühner - kuehner@cl.uni-heidelberg.de


## Acknowledgments

* Special thanks to @debjitpaul who took his time for zoom calls with us and helped us working through error messages and getting his program started.

## Reference
@inproceedings{paul-frank-2019-ranking,
    title = "Ranking and Selecting Multi-Hop Knowledge Paths to Better Predict Human Needs",
    author = "Paul, Debjit  and
      Frank, Anette",
    booktitle = "Proceedings of the 2019 Conference of the North {A}merican Chapter of the Association for Computational Linguistics: Human Language Technologies, Volume 1 (Long and Short Papers)",
    month = jun,
    year = "2019",
    address = "Minneapolis, Minnesota",
    publisher = "Association for Computational Linguistics",
    url = "https://www.aclweb.org/anthology/N19-1368",
    pages = "3671--3681",
    abstract = "To make machines better understand sentiments, research needs to move from polarity identification to understanding the reasons that underlie the expression of sentiment. Categorizing the goals or needs of humans is one way to explain the expression of sentiment in text. Humans are good at understanding situations described in natural language and can easily connect them to the character{'}s psychological needs using commonsense knowledge. We present a novel method to extract, rank, filter and select multi-hop relation paths from a commonsense knowledge resource to interpret the expression of sentiment in terms of their underlying human needs. We efficiently integrate the acquired knowledge paths in a neural model that interfaces context representations with knowledge using a gated attention mechanism. We assess the model{'}s performance on a recently published dataset for categorizing human needs. Selectively integrating knowledge paths boosts performance and establishes a new state-of-the-art. Our model offers interpretability through the learned attention map over commonsense knowledge paths. Human evaluation highlights the relevance of the encoded knowledge.",
}

