# Project Proposal

## Summary

  - [Motivation](#motivation)
  - [Basis of the project](#basis-of-the-project)
  - [Approach](#approach)
  - [Dataset](#dataset)
  - [Tools](#tools)
  - [Goals](#goals)


## Motivation

In the field of computational linguistics, stance detection is an established task. It is about extracting different motives behind arguments and statements. However, stances themselves often do not provide enough information about the background of a statement. What could counteract this problem is the assignment of human needs. <br>

> "Human needs are commonly used to refer to the drivers of peoples’ actions, the motives behind human behavior. Other uses of the concept include needs as instruments to achieve a certain goal and needs as societal requirements to flourish or experience a good life (Gasper, 2004). Needs as motives imply that all human actions can be understood as originating in the quest for satisfying or actualizing needs. Needs as societal requirements concern what is necessary for people to avoid serious harm and participate in their society, whether or not they are mobilized by them. Needs as requirements to achieve a certain outcome like fulfillment, happiness, or satisfaction fall in between the two last explanations. They take the elements that contribute to achieving the goal of a good life" ~ quoted from [here](https://link.springer.com/referenceworkentry/10.1007%2F978-94-007-0753-5_1345) 

In our project we want to use the concept of human needs to better analyze text content. The human needs assignment to texts allows a detailed analysis of the motivations and human needs from which people argue. With this gained information, many aspects of text analysis can be considered in a much more detailed and profound way.

## Basis of the project
In [Paul&Frank(2019)](https://www.aclweb.org/anthology/N19-1368/) this approach is already taken up. Here, the two authors pursue the question why certain positions, opinions and views on a topic are expressed by a person. The authors see the underlying reason in the concepts of human needs, which has been established by Maslow(1943) as the hierarchy of needs and can be extended by Reiss(2004).
> Maslow's hierarchy of needs is a motivational theory in psychology comprising a five-tier model of human needs, often depicted as hierarchical levels within a pyramid. From the bottom of the hierarchy upwards, the needs are: physiological (food and clothing), safety (job security), love and belonging needs (friendship), esteem, and self-actualization. ~ quoted from [here](https://www.simplypsychology.org/maslow.html#gsc.tab=0)

Reiss takes up this pyramid of Maslow and expands it with further, more profound motives, matching the respective categories (see picture below). <br>
Based on Maslow's pyramid and looking at the concepts from [ConceptNet](https://conceptnet.io/), we can then target the assignment and analyze and evaluate the results. By constructing subgraphs and knowledge paths for each sentence, we try to assign the best fitting human need to each sentence.

<p align="center">
  <img src="pictures/maslows-hierarchy.png" />
</p>

[Paul&Frank(2019)](https://www.aclweb.org/anthology/N19-1368/)s approach is based on narrative texts. As their dataset they use the ROCStories dataset ([Mostafazadeh, 2016]()) which contains a collection of narrative texts. We extend this approach by looking at argumentative texts and extending the procedure for our usage.

## Approach
First of all, we look for a suitable [data set](#dataset). It is important here that these are argumentative texts. The selected data set must then be prepared according to the format required by the model. To do this, we begin by manually annotating all of the four hundred essays from our data set with one of the maslow and one of the reiss motives. <br> After having done that we use the fleiss kappa score to calculate the Inter Annotator Agreement. To produce a gold standard, each of us annotated an additional 75 essays, resulting in 100 essays total wich have been annotated by all 4 members.  *E.g. If I annotated essays 1-101 in the first pass, I will now annotate the first 25 of each of 102-202, 203-303, and 304-404.* <br> Since our selected data was already seperated into train and test, we only had to put our train and test files into correct format (see files attached). <br> For our project we only looked at the last paragraph of each essay, as this was the closing argument and provided a good overview of the opinions and standpoints of the author (more details under [data set](#dataset)). For this, we selected the last paragraph of each essay of our dataset using *Comparer.py* and compared each word in it with the concepts from ConceptNet to generate a list of concepts for the sentence. <br> After doing that we started with the steps equivalent to [this](https://github.com/debjitpaul/Multi-Hop-Knowledge-Paths-Human-Needs). Because some of the steps and attached files from Debjit Pauls github were not working for us (see project_report, problems) we changed a few things, which can be taken from our README.md. After constructing the subgraphs for every sentece and extracting the relevant knowledge paths, we can extract the human needs from the created knowledge paths and assign them to the essays (*Human_needs_assigner.py*). <br> The last thing we do is evaluate and assess our obtained results. <br> For textual analysis we wanted to use OpenFraming. But unfortunately the tool is still not working to date (25th march 2021) - due to an internal server error.

## Dataset

We chose the Student essay corpus as our data set. The dataset is provided by the [TU Darmstadt](https://www.tu-darmstadt.de/)
<br>
[This](https://www.informatik.tu-darmstadt.de/ukp/research_6/data/argumentation_mining_1/argument_annotated_essays_version_2/index.en.jsp) is the version we used for our project. <br> The dataset consist of 404 argumentative essays covering a wide array of topics. Each essay has a different topic and ends with a summary paragraph. Since this last paragraph contains all the necessary information about the text, we consider only that one in our project. The essays are already split by the editor into train and test.

## Tools
#### ConceptNet
> ConceptNet is a freely-available semantic network, designed to help computers understand the meanings of words that people use
ConceptNet originated from the crowdsourcing project Open Mind Common Sense, which was launched in 1999 at the MIT Media Lab. It has since grown to include knowledge from other crowdsourced resources, expert-created resources, and games with a purpose. ~ quoted from [conceptnet](https://conceptnet.io/)

#### OpenFraming
> OpenFraming can perform two types of computational framing analysis: 1) unsupervised topic modeling based on Latent Dirichlet Allocation (LDA; Blei et al. (2003)), and 2) supervised learning using deep neural network Bidirectional Encoder Representations from Transformers (BERT; Devlin et al. (2018)) ~ quoted from [openframing](https://arxiv.org/pdf/2008.06974.pdf)

## Goals
The aim of the project is to develop a method with which human needs can also be assigned to argumentative texts and thus to carry out a meaningful analysis.   
