# Project Report

As already mentioned in project_proposal, our goal of the project was to enable a meaningful mapping of human needs on argumentative data using the concept net graphs. In this report we will present the results of our project and describe problems encountered and findings made.

## Summary

  - [Project summary](#project-summary)
  - [Results and Evaluation](#results-and-evaluation)
  - [The tools and how they worked](#the-tools-and-how-they-worked)
  - [Problems that accured](#problems-that-accured)
  - [Lessons learned](#lessons-learned)
 


## Project summary

In this project, we followed up on the paper by [Paul&Frank(2019)](https://www.aclweb.org/anthology/N19-1368.pdf). While the aforementioned paper worked on narrative texts, we extended this approach to argumentative texts. For this, we used [this](https://github.com/debjitpaul/Multi-Hop-Knowledge-Paths-Human-Needs) approach provided by @DebjitPaul, and adapted it to our project. 

## Results and Evaluation
#### Annotations and Maslow hierarchy of needs
Our first step in this project, as mentioned earlier, was to manually annotate all essays. Here we noticed that the Maslow categories exclusively represent human needs. However, we also found some essays that were not about humans, but about the needs of other living or non-living beings. For our project we decided to extend Maslow's motives to other, non-human existences. <br> In principle however we would find a revised, more existence-needs integrating pyramid meaningful. Above all in today's time, where animal protection, nature protection etc. are large important topics, which also is much written and reported about, it is important that these needs are filtered out and included. <br> **The only question left when revising the Maslow pyramind is: would one like to regard e.g. the animal protection as human need - thus as need of humans to protect the environment or as the need of the animal to live?** 
<br>
Another observation we made during our annotation was that the Maslow and Reiss categories are not so simple and not clearly assignable. Already during the annotation we all noticed that often more than one category is applicable. While we decided on the basis of our gut feeling what was most applicable for us, we realized already when comparing with the others that in some examples we had even assigned four different needs. And each one fit in its own way. Then, when we used Fleiss Kappa to calculate our inter-annotator agreement, we found that we came up with an agreement of about 0.7 for Maslow and 0.4 for Reiss. This result made it clear that even if we interpreted the human needs behind an essay in the same way, it would be difficult to agree on the Reiss motives, which are even more concrete than the Maslow motives. Which motives one suspects behind a text written by someone else depends too much on the person reading it and his personal feelings.
<br>
After extracting the knowledge paths, we found that ConceptNet also has a problem with assigning Reiss motives. While the Maslow motives were often well assigned, ConceptNet showed a clear preference for some motives of Reiss. For example, the Reiss motive social was assigned much more often and also in places where a person would have already tended towards love/belonging. Thus ConceptNet has difficulties distinguishing social from concrete feelings such as love.

#### Assigning the human needs
After extracting the knowledgepaths, we found that the neural model of @DebjitPaul was not immediately executable without errors. Again there were a lot of version problems, the versions given in the [github](https://github.com/debjitpaul/Multi-Hop-Knowledge-Paths-Human-Needs) are outdated and every version change caused a problem with another package (see [Problems that accured](#problems-that-accured)). Since we didn't want to invest all our time again in finding out the correct versions of another project and @debjitpaul couldn't help with the faulty version specifications either, we decided to develop our own method for this. Since the knowledgepaths are already returned according to their expressiveness, we applied them in *humans_needs_assginer.py* by using and assigning the human needs of the first path we found. 
<br>
 
#### Evaluation of results
After the human needs have been successfully assigned for all essays, we evaluate the whole thing. In *human_needs_evaluation.py" we have written a code for evaluation. Here we calculate Precision, Accuracy, Recall and F1-Score and with these measurements we analyze the evaluation. Concrete comments and visualisations of the evaluation can be found in the jupyter notebook. 

<p align="center">
  <img src="pictures/Accuracy.png" />
</p>

> The system scored an accuracy of roughly 28% for Maslow categories, while scoring merely 7.7% for Reiss motives. Just as we had conjectured at the beginning of our project (and also experienced ourselves during the Inter-Annotor-Agreement phase), the assigment of Maslow categories is significantly easier for both the system and us human. This is due to the more coarse-grained nature of the Maslow categories, which only consist of 5 categories and thus leavey the system and us human less options to choose from. With a broad spectrum of possibile categories to choose from, the system struggles more and has significant difficulties to assign the correct Reiss motive (though it did assign the correct Maslow category for many examples). <br> The more fine-grained nature of the Reiss motives provide much more options, some of which correspond heavily to one another, which makes the assigment of a correct Reiss motive even more difficult. This confirms our study conjectures at the beginning: <br> **The classification of more coarse-grained categories works signiifcantly better than that of fine-grained categories.** 
>>Though the results may seem disappointing at first sight, we're still somewhat satisfied with them as we had to use a novel, more naive approach, which was still effective in assigning human needs. For the accurate consideration of our results, we also need to take the nature of our texts into account. Entire paragraphs provide much more tokens and thus also much more information, which also makes them more difficult to process. Another factor that contributes to the difficulty of these texts is that they are all of varying lengths. Still, at least for Maslow, the system was able to score satisfying results.


<p align="center">
  <img src="pictures/Recall_test.png" />
</p>

> The system scored a macro-averaged recall of roughly 20% for Maslow categories, while scoring just 3.54% for Reiss motives. Again, these values for the recall metric confirm our study conjecture that fine-grained categories like Reiss motives leave the system too much options to choose from, which lowers the system's performance and confidence significantly. As mentioned before, another contributing factor to this is that some Reiss motives are heavily connected to each other and could possibly be seen as interchangeable, which explains the low results for recall. Of particular interest here are the Reiss motives "order" and "curiosity": motive "order" had a precision of 50% but a recall value of only 7%, meaning that the system struggled to retrieve the relevant examples for this class. For "curiosity", the opposite is true, as the system scored a precision of 16% but a recall of 25%. Given a much smaller range of options, the system is much more confident and thus provides even better performance for Maslow categories. Especially "spiritual growth" and "stability" provide good results here with 38% each, which is not suprising given that they are the most frequently assigned Maslow categories in our gold annotations. However, classes which are assigned with an insignificant frequency in the gold data perform poorly, as is visible by the results for the class "physiological needs" (and its corresponding Reiss motives).
>> Again, the results may seem insatisfactory, but considering aforementioned nature of the texts, problems regarding lack of frequency in our gold data and our novel approach, the results again are somewhat acceptable. Just like for precision, a larger and, crucially, more balanced corpus could potentially improve these results.

<p align="center">
  <img src="pictures/Precision_test.png" />
</p>

> The system scored a macro-averaged precision of roughly 19% for Maslow categories, while scoring only 5.4% for Reiss categories. Yet again the system struggles much more with the classification of the correct Reiss motive due to their fine-grained nature. The possibility to choose from a much wider range of possible classes makes negative impacts on the systems performance. Again, nature of the Reiss motives too negatively influences the system's performance due to aforementioned reasons. However, provided with a much more limited variety of classes to choose from, the system's performance increases heavily, as is visible in the precision scores of Maslow categories. It is interesting to take a quick glance at the scores for each class: As we had expected, classes that are barely seen in the gold data produce poor results due to their lack of generalization. In our case, "physiological needs" scored 0% precision because of this. On the other hand, classes which are more frequent in the gold data score much better results, as visible by the results for the classes "spiritual growth" and "stability". These results confirm our study conjectures about the frequency of certain classes, in which frequent classes scored significantly higher than classes which are barely existent in the gold data. Classes in the middle of this range provide expected results, in which they don't score very high but also not dramatically low.
>> Though the results may again seem utterly dissatisfying and not quite what we had hoped for, it is still somewhat accaptable due to the more difficult and tricky nature of our texts and the novel approach we had to take. The problems that arise from the lack of gold data for certain classes (i.e. "physiological needs") will have to be mitigated either way. A more balanced distribution of texts could possibly solve this problem, but the content of the corpus at use could make this approach quite difficult, as it was the case.

<p align="center">
  <img src="pictures/F1_test.png" />
</p>

> The system scored a macro-averaged F1 score of roughly 19% for Maslow categories and only 3.5% for Reiss categories. Considering the previously discussed precision and recall measures, these results seem fitting, as the F1 score combines precision and recall and thus serves as a harmonic mean between both measures.
>> The F1 score too may not seem satisfactory, but considering all the circumstantances and factors of the study, F1 score too is still somewhat acceptable, especially for Maslow categories.

<p align="center">
  <img src="pictures/Classification_Report_test.png" />
</p>

> Matrix 1
<p align="center">
  <img src="pictures/Matrix_test_1.png" />
</p>

> Matrix 2
<p align="center">
  <img src="pictures/Matrix_test_2.png" />
</p>

> Matrix 3
<p align="center">
  <img src="pictures/Matrix_test_3.png" />
</p>

> Matrix 4
<p align="center">
  <img src="pictures/Matrix_4_test.png" />
</p>




## The Tools and how they worked
### Problems that accured

> *DISCLAIMER :* The problems with the tools described below are not intended to apportion blame or condemnation. The problems we have encountered are simply part of our project process and the reason for our changes and innovations. 

As already mentioned we used [this](https://github.com/debjitpaul/Multi-Hop-Knowledge-Paths-Human-Needs) approach provided by @DebjitPaul. Unfortunately, we had some difficulties with the execution of the given code. <br>
Probably the biggest problem was the outdated version information. While we repeatedly encountered the statement to always use the versions of packages that a publisher himself  specifies, in this case we were constantly confronted with error messages. Every time we worked through an error message and reset versions, we sooner or later came to another error message about faulty package version. <br>
In the end, we found that the following complications were to blame: <br>
- the tensorflow version 1.12.0 as recommended in given [github](https://www.aclweb.org/anthology/N19-1368.pdf) does not support tensorflow-hub
- only tensorflow 1.15.0 and upwards supports tensorflow-hub
- python version 3.8 only supports tensorflow versions higher then 2.0
- python version 3.6 was required
- the requirement of keras was not mentioned anywhere, but is a necessity for the execution of the code
- keras version has to be 2.2.4 because everything higher upgrades tensorflow automatically to tensorflow 2.0 
  while anything lower does not support certain modules needed for the code 
- and tensorflow 2.0 has so many technical changes, that it is not compatible with our models 
> The final versions that worked for us can be found in README.md
<br> 
All the above mentioned points had to be determined and improved in the course of development. This very often set us back in the overall process. <br> 
<br>
Unfortunately, there were also problems with the attached files in the [github](https://github.com/debjitpaul/Multi-Hop-Knowledge-Paths-Human-Needs). For example, we kept getting a namespace error "'Namespace' object has no attribute 'txtfile'" for weeks which prevented us from executing the code, and  we could never get rid of it completely. In the Zoom meetings with @debjitpaul we addressed this problem until it turned out that there was a typo in his original file which was responsible for the error. Instead of calling a "txtfile" it should have been "inputfile". We are aware that these are minor issues. However, this accumulation of problems led to significant time losses in our project and we had to put in more time in the last weeks than we had planned for.
<br> 
<br> 
Due to a lack of capacity on our own computers, we turned to our institute's technology group for cluster access. They got back to us and granted us access. After everything went well here at first, we again reached a point where we lacked the necessary rights to further execute our project while also encountering a number or errors. Unfortunately, despite contacting them several times, we finally did not hear back from the technology group. Because of this, we had to work on our own computers again and start from where we left off. Despite the fact that we divided it among ourselves, the program needed an average of about one hour per sentence to construct a subgraph, while especially long sentences with big context lists took up to 3 hours. Having 400 essays to work through, it took us quite a long time. 
<br> 
<br>
As already mentioned briefly in our project_proposal we wanted to do a textual analysis with the OpenFraming tool. Again, we had already prepared our data to match the model and were ready to work with the tool. Unfortunately, we received an error message right at the first step. After contacting the OpenFraming team, they initially suspected that the error must lie with us. Therefore, they gave us instructions on what the files should look like (csv file, comma separated). But despite several attempts to change the files, the error could not be solved. After weeks, the OpenFraming team determined that it was an internal error that only they could fix. However, they soon realized that it was a slightly bigger problem and offered us the alternative of working with docker files. But even here there were several errors on their part, which is why we no longer got to work with their tool. Although several members of the OpenFraming team contacted us by mail afterwards and let us know that they were working on it, they broke off contact at some point. Unfortunately, the tool is still not working till date (25th march 2021). Of course we don't blame anyone for this. We are well aware that everyone has their own work to do. We only regret that we had invested so much time and effort into the tool and in the end we could not work with it. The textual analysis complementary to our obtained results would have been certainly very interesting.


### Lessons learned

Encountering all these problems taught us many things during the timespan of this software project. 
<br> 
First, we learned that a project dependent on other tools is always risky. While this time we put a lot of effort into a tool that ultimately didn't work, next time we would look for alternatives much earlier and consider other options. Our somewhat too great optimism at the beginning, that everything will work again, was reduced from error message to error message. 
<br>

We also learned how important communication is when working with foreign code and tools. While we hesitated at the beginning, we now immediately contacted the editors with every problem where we did not know what to do. Even if this meant that we had to write them emails several times a week until we got an answer. 
<br> 

We also learned that error messages are not mistakes but part of the process. In this software project we encountered more errors than in our entire studies, and yet we found a solution every time. 
<br>

This leads us to the next point. The more error messages there were in the beginning, the lower our motivation became. Especially in such moments where it seemed to run partly not at all, we learned how important it is to stay on. Also the importance of cooperation in a group became clearer from week to week. Despite the fact that we already functioned well as a group from the beginning and had no problem with work sharing, each error message welded us together a bit more. There were many days when we spent hours together trying to solve problems and implement new ideas. Despite not being able to meet in person due to Corona times, we found a way to communicate and act as a group on a daily basis. 
<br> 

If we were to start the same project again from scratch, we would probably approach things differently. However, we would not have made these experiences, which we will take with us for our future projects.

